﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace prohlizec
{

    public partial class frmDirectories : Form
    {
        private struct ImageInfo
        {
            public Size resolution;
            public int index;
            public long filesize;
            public DateTime time;
        }

        private List<string> files = new List<string>();

        public frmDirectories(String nfolder)
        {
            InitializeComponent();
            //nacteme pouzivane logicke jednotky
            GetDrives(treeDirectories.Nodes);
            //pokud byla zadana slozka v parametru, tak najedeme primo do ni
            if (Directory.Exists(nfolder))
            {
                string drive = nfolder.Substring(0, 3);
                TreeNode node = new TreeNode();
                foreach (TreeNode n in treeDirectories.Nodes)
                    if (n.Text == drive)
                        node = n;
                nfolder = nfolder.Substring(3);
                while (nfolder.Length > 0)
                {
                    int x = nfolder.IndexOf("\\");
                    if (x > 0)
                    {
                        node = node.Nodes.Add(nfolder.Substring(0, x));
                        nfolder = nfolder.Substring(x);
                    }
                    else
                    {
                        node = node.Nodes.Add(nfolder);
                        nfolder = "";
                    }
                }
                treeDirectories.SelectedNode = node;
                treeDirectories_BeforeSelect(null, new TreeViewCancelEventArgs(node, false, TreeViewAction.ByKeyboard));
            }
        }

        private string GetFileSize(long Bytes)
        {
            if (Bytes >= 1073741824)
            {
                Decimal size = Decimal.Divide(Bytes, 1073741824);
                return String.Format("{0:##.##} GB", size);
            }
            else if (Bytes >= 1048576)
            {
                Decimal size = Decimal.Divide(Bytes, 1048576);
                return String.Format("{0:##.##} MB", size);
            }
            else if (Bytes >= 1024)
            {
                Decimal size = Decimal.Divide(Bytes, 1024);
                return String.Format("{0:##.##} KB", size);
            }
            else if (Bytes > 0 & Bytes < 1024)
            {
                Decimal size = Bytes;
                return String.Format("{0:##.##} Bytes", size);
            }
            else
            {
                return "0 Bytes";
            }
        } 

        private TreeNode Node(string path, string text, string type)
        {
            //ke kazdemu adresari pridame "dummy" uzel, aby to slo rozbalit
            TreeNode n = new TreeNode(text);
            n.Nodes.Add("");
            return n;
        }

        private void GetDrives(TreeNodeCollection nodes)
        {
            string[] drives = Directory.GetLogicalDrives();
            foreach (string p in drives)
                nodes.Add(Node("", p, "folder"));
        }

        private void GetFolders(string path, TreeNodeCollection nodes)
        {
            try
            {
                string[] dirs = Directory.GetDirectories(path);
                foreach (string p in dirs)
                    nodes.Add(Node("", p.Substring(path.Length), "folder"));
            }
            catch (Exception)
            { }
        }

        private string GetPath(TreeNode node)
        {
            //z uzlu vytvori string cele cesty
            TreeNode aktualni = node;
            string path = "";
            while (aktualni != null)
            {
                if (!(aktualni.Text.EndsWith("\\")))
                    path = "\\" + path;
                path = aktualni.Text + path;
                aktualni = aktualni.Parent;
            }
            return path;
        }

        private void treeDirectories_BeforeExpand(object sender, TreeViewCancelEventArgs e)
        {
            //pri rozbaleni uzlu najde vsechny podslozky
            e.Node.Nodes.Clear();
            GetFolders(GetPath(e.Node), e.Node.Nodes);
        }

        private void treeDirectories_BeforeSelect(object sender, TreeViewCancelEventArgs e)
        {
            panelThumbnails.Controls.Clear();
            treeDirectories.Enabled = false;
            try
            {
                //nacte soubory v adresari
                string[] nfiles = Directory.GetFiles(GetPath(e.Node));
                files.Clear();
                int x = 0, y = 0;
                int index = 0;
                foreach (string file in nfiles)
                {
                    string f = file.ToLower();
                    if (f.EndsWith(".jpg") || f.EndsWith(".jpeg"))
                    {
                        //pokud je to obrazek, vytvori novy PictureBox na formulari
                        PictureBox picture = new PictureBox();
                        picture.Size = new Size(160, 120);
                        picture.BorderStyle = BorderStyle.Fixed3D;
                        picture.Click += Thumbnail_Click;
                        picture.DoubleClick += Thumbnail_DoubleClick;
                        picture.Left = x;
                        picture.Top = y;
                        x += 160;
                        if (x + 160 > panelThumbnails.Width)
                        {
                            x = 0;
                            y += 120;
                        }
                        Image img = Image.FromFile(file);
                        ImageInfo info = new ImageInfo();
                        FileInfo fileinfo = new FileInfo(file);
                        info.index = index++;
                        info.resolution = img.Size;
                        info.filesize = fileinfo.Length;
                        info.time = fileinfo.CreationTime;
                        picture.Tag = info;
                        if (img.Width > img.Height)
                        {
                            int height = 160 * img.Height / img.Width;
                            picture.Image = img.GetThumbnailImage(160, height, null, new System.IntPtr());
                        }
                        else
                        {
                            int width = 120 * img.Width / img.Height;
                            picture.Image = img.GetThumbnailImage(width, 120, null, new System.IntPtr());
                        }
                        panelThumbnails.Controls.Add(picture);
                        files.Add(file);
                        //zajistuje, ze uzivatel muze rovnou klikat na obrazky a pouzivat je, i kdyz se ostatni jeste nacitaji
                        Application.DoEvents();
                    }
                }
            }
            catch (Exception)
            { }
            treeDirectories.Enabled = true;
        }

        private void panelThumbnails_Resize(object sender, EventArgs e)
        {
            //premisteni obrazku pri zmene velikosti formulare
            int x = 0, y = 0;
            foreach (Control picture in panelThumbnails.Controls)
            {
                picture.Left = x;
                picture.Top = y;
                x += 160;
                if (x + 160 > panelThumbnails.Width)
                {
                    x = 0;
                    y += 120;
                }
            }
        }

        private void Thumbnail_Click(object sender, EventArgs e)
        {
            //zobrazeni informaci o obrazku
            PictureBox selected = (PictureBox)sender;
            txtFilename.Text = files[((ImageInfo)(selected.Tag)).index];
            txtSize.Text = GetFileSize(((ImageInfo)(selected.Tag)).filesize);
            txtResolution.Text = ((ImageInfo)(selected.Tag)).resolution.Width.ToString() + " x " + ((ImageInfo)(selected.Tag)).resolution.Height.ToString();
            txtTime.Text = ((ImageInfo)(selected.Tag)).time.ToString();
        }

        private void Thumbnail_DoubleClick(object sender, EventArgs e)
        {
            //prechod do rezimu zobrazeni obrazku
            frmView form = new frmView(files, ((ImageInfo)((PictureBox)sender).Tag).index);
            form.ShowDialog();
        }
    }
}
