﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace prohlizec
{
    public partial class frmView : Form
    {
        private List<string> files;
        private int index;
        private int zoom;
        private int zoomopt;
        private bool slideshow = false;
        private int oldindex;
        private Color oldcolor;

        public void ZoomOptimal()
        {
            //vypocita zoom optimalni pro zobrazeni na celou obrazovku
            if (((Image)(picture.Tag)).Width > ((Image)(picture.Tag)).Height)
            {
                int width = Screen.PrimaryScreen.WorkingArea.Width - 28;
                zoom = zoomopt = 100 * width / ((Image)(picture.Tag)).Width;
            }
            else
            {
                int height = Screen.PrimaryScreen.WorkingArea.Height - 32;
                zoom = zoomopt = 100 * height / ((Image)(picture.Tag)).Height;
            }
        }

        public Bitmap ResizeBitmap(Bitmap b, int nWidth, int nHeight)
        {
            //zmeni rozmery obrazku
            Bitmap result = new Bitmap(nWidth, nHeight);
            using (Graphics g = Graphics.FromImage((Image)result))
                g.DrawImage(b, 0, 0, nWidth, nHeight);
            return result;
        }

        private Bitmap MakeGrayscale(Bitmap original)
        {
            //vytvori cernobilou variantu obrazku
            Bitmap newBitmap = new Bitmap(original.Width, original.Height);

            for (int i = 0; i < original.Width; i++)
            {
                for (int j = 0; j < original.Height; j++)
                {
                    //veme pixel z obrazku
                    Color originalColor = original.GetPixel(i, j);

                    //vytvori cernobilou barvu pixelu
                    int grayScale =
                       (int)((originalColor.R * .3) +
                              (originalColor.G * .59) +
                              (originalColor.B * .11));
                    Color newColor =
                       Color.FromArgb(grayScale, grayScale, grayScale);

                    //ulozi pixel do noveho obrazku
                    newBitmap.SetPixel(i, j, newColor);
                }
            }

            return newBitmap;
        }
        private string GetFileSize(long Bytes)
        {
            //prevadi velikosti na "human readable"
            if (Bytes >= 1073741824)
            {
                Decimal size = Decimal.Divide(Bytes, 1073741824);
                return String.Format("{0:##.##} GB", size);
            }
            else if (Bytes >= 1048576)
            {
                Decimal size = Decimal.Divide(Bytes, 1048576);
                return String.Format("{0:##.##} MB", size);
            }
            else if (Bytes >= 1024)
            {
                Decimal size = Decimal.Divide(Bytes, 1024);
                return String.Format("{0:##.##} KB", size);
            }
            else if (Bytes > 0 & Bytes < 1024)
            {
                Decimal size = Bytes;
                return String.Format("{0:##.##} Bytes", size);
            }
            else
            {
                return "0 Bytes";
            }
        } 

        private void LoadIndex()
        {
            //zobrazi obrazek zadaneho cisla (globalni promenna index)
            picture.Tag = picture.Image = Image.FromFile(files[index]);
            ZoomOptimal();
            ZoomChange();

            FileInfo fileinfo = new FileInfo(files[index]);
            lblFilename.Text = files[index];
            lblSize.Text = GetFileSize(fileinfo.Length);
            lblTime.Text = fileinfo.CreationTime.ToString();
            lblIndex.Text = (index + 1).ToString() + "/" + files.Count.ToString();
        }

        private void ZoomChange()
        {
            //zvetsi/zmensi obrazek podle globalni promenne zoom
            Image img = (Image)picture.Tag;
            picture.Image = ResizeBitmap((Bitmap)img, img.Width * zoom / 100, img.Height * zoom / 100);
            picture.Size = picture.Image.Size;
            lblZoom.Text = "Zoom: " + zoom.ToString() + "%";
        }

        public frmView(List<string> nfiles, int nindex)
        {
            InitializeComponent();
            files = nfiles;
            index = nindex;
            LoadIndex();
        }

        private void toolbtnFirst_Click(object sender, EventArgs e)
        {
            index = 0;
            LoadIndex();
        }

        private void toolbtnPrevious_Click(object sender, EventArgs e)
        {
            if (index > 0)
            {
                --index;
                LoadIndex();
            }
        }

        private void toolbtnNext_Click(object sender, EventArgs e)
        {
            if (index < files.Count - 1)
            {
                ++index;
                LoadIndex();
            }
        }

        private void toolbtnLast_Click(object sender, EventArgs e)
        {
            index = files.Count - 1;
            LoadIndex();
        }

        private void toolbtnZoomPlus_Click(object sender, EventArgs e)
        {
            zoom += 5;
            ZoomChange();
        }

        private void toolbtnZoomMinus_Click(object sender, EventArgs e)
        {
            if (zoom > 5)
            {
                zoom -= 5;
                ZoomChange();
            }
        }

        private void picture_DoubleClick(object sender, EventArgs e)
        {
            this.Close();
        }

        private void frmView_KeyUp(object sender, KeyEventArgs e)
        {
            if (slideshow)
            {
                if (e.KeyCode == Keys.Escape)
                {
                    slideshow = false;
                    timerSlideshow.Enabled = false;
                    toolbar.Visible = true;
                    statusStrip1.Visible = true;
                    this.BackColor = oldcolor;
                    index = oldindex;
                    LoadIndex();
                }
            }
            else
            {
                switch (e.KeyCode)
                {
                    case Keys.Home:
                        toolbtnFirst_Click(sender, e);
                        break;
                    case Keys.End:
                        toolbtnLast_Click(sender, e);
                        break;
                    case Keys.PageUp:
                        toolbtnPrevious_Click(sender, e);
                        break;
                    case Keys.PageDown:
                        toolbtnNext_Click(sender, e);
                        break;
                    case Keys.Add:
                        toolbtnZoomPlus_Click(sender, e);
                        break;
                    case Keys.Subtract:
                        toolbtnZoomMinus_Click(sender, e);
                        break;
                    case Keys.Multiply:
                        zoom = 100;
                        ZoomChange();
                        break;
                    case Keys.Divide:
                        ZoomOptimal();
                        ZoomChange();
                        break;
                    case Keys.Enter:
                        picture_DoubleClick(sender, e);
                        break;
                    case Keys.Escape:
                        Application.Exit();
                        break;
                    case Keys.G:
                        toolbtnGreyscale_Click(sender, e);
                        break;
                    case Keys.L:
                        toolbtnRotateLeft_Click(sender, e);
                        break;
                    case Keys.R:
                        toolbtnRotateRight_Click(sender, e);
                        break;
                    case Keys.H:
                        toolbtnFlipH_Click(sender, e);
                        break;
                    case Keys.V:
                        toolbtnFlipV_Click(sender, e);
                        break;
                    case Keys.S:
                        toolbtnSlideshow_Click(sender, e);
                        break;
                }
            }
        }

        private void toolbtnQuit_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void toolbtnRotateLeft_Click(object sender, EventArgs e)
        {
            ((Image)(picture.Tag)).RotateFlip(RotateFlipType.Rotate270FlipNone);
            ZoomChange();
        }

        private void toolbtnGreyscale_Click(object sender, EventArgs e)
        {
            picture.Tag = MakeGrayscale((Bitmap)((Image)(picture.Tag)));
            ZoomChange();
        }

        private void toolbtnRotateRight_Click(object sender, EventArgs e)
        {
            ((Image)(picture.Tag)).RotateFlip(RotateFlipType.Rotate90FlipNone);
            ZoomChange();
        }

        private void toolbtnFlipH_Click(object sender, EventArgs e)
        {
            ((Image)(picture.Tag)).RotateFlip(RotateFlipType.RotateNoneFlipX);
            ZoomChange();
        }

        private void toolbtnFlipV_Click(object sender, EventArgs e)
        {
            ((Image)(picture.Tag)).RotateFlip(RotateFlipType.RotateNoneFlipY);
            ZoomChange();
        }

        private void timerSlideshow_Tick(object sender, EventArgs e)
        {
            if (index == files.Count - 1)
            {
                slideshow = false;
                timerSlideshow.Enabled = false;
                toolbar.Visible = true;
                statusStrip1.Visible = true;
                this.BackColor = oldcolor;
                index = oldindex;
                LoadIndex();
            }
            else
                toolbtnNext_Click(sender, e);
        }

        private void toolbtnSlideshow_Click(object sender, EventArgs e)
        {
            slideshow = true;
            toolbar.Visible = false;
            statusStrip1.Visible = false;
            oldindex = index;
            oldcolor = this.BackColor;
            this.BackColor = Color.Black;
            index = 0;
            LoadIndex();
            timerSlideshow.Enabled = true;
        }
    }
}
